package treeplus;

import java.awt.Color;
import java.awt.geom.Rectangle2D;
import java.io.File;
import java.util.ArrayList;
import java.util.Queue;
import java.util.Random;
import java.util.concurrent.LinkedBlockingQueue;

import javax.swing.BorderFactory;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JScrollPane;

import org.jgraph.JGraph;
import org.jgraph.graph.DefaultEdge;
import org.jgraph.graph.DefaultGraphCell;
import org.jgraph.graph.DefaultGraphModel;
import org.jgraph.graph.DefaultPort;
import org.jgraph.graph.GraphConstants;
import org.jgraph.graph.GraphModel;

import treeplus.tree.BFS;
import treeplus.tree.TreePlusGraph;
import treeplus.ui.MainWindow;
import treeplus.util.GraphLoader;

public class TestMain {

	public static void main(String[] args) {

		// Construct Model and Graph
		GraphModel model = new DefaultGraphModel();
		JGraph graph = new JGraph(model);

		// Control-drag should clone selection
		graph.setCloneable(true);

		// Enable edit without final RETURN keystroke
		graph.setInvokesStopCellEditing(true);

		// When over a cell, jump to its default port (we only have one, anyway)
		graph.setJumpToDefaultPort(true);

		// Insert all three cells in one call, so we need an array to store them
		DefaultGraphCell[] cells = new DefaultGraphCell[7];

		// Create Startpunkt
//		 cells[0] = createVertex("Wien");
//		 cells[1] = createVertex("Salzburg");
//		 cells[2] = createVertex("Haag City");
//		 cells[3] = createVertex("Leonding");
//		
//				
//		// // Create Edge
//		 cells[4] = createEdge(cells[0], cells[1]);
//		 cells[5] = createEdge(cells[0], cells[2]);
//		 cells[6] = createEdge(cells[1], cells[3]);
//		//
		// System.out.println("roots: " +model.getRootCount());
		JFrame frame = new JFrame();
		cells = doTheMetal(frame, graph);
		// Set Arrow Style for edge
		// int arrow = GraphConstants.ARROW_TECHNICAL;
		// GraphConstants.setLineEnd(edge.getAttributes(), arrow);
		// GraphConstants.setEndFill(edge.getAttributes(), true);

		// Insert the cells via the cache, so they get selected
		graph.getGraphLayoutCache().insert(cells);

		// Show in Frame

		frame.getContentPane().add(new JScrollPane(graph));
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.pack();
		frame.setVisible(true);
	}

	public static DefaultGraphCell[] doTheMetal(JFrame frame, JGraph jgraph) {
		JFileChooser chooser = new JFileChooser();
		chooser.setMultiSelectionEnabled(true);
		int option = chooser.showOpenDialog(frame);
		TreePlusGraph treePlusGraph = null;
		
		if (option == JFileChooser.APPROVE_OPTION) {
			File sf = chooser.getSelectedFile();
			String filelist = "nothing";

			if (sf.canRead())
				filelist = sf.getName();

			 treePlusGraph = GraphLoader.loadGraph(sf);
		}

		Queue queue = new LinkedBlockingQueue();
		
		int[][] graph = treePlusGraph.getMatrix();
		ArrayList names = treePlusGraph.getNames();
		int size = names.size();
		boolean[] visited = new boolean[size]; // init false

		int curRow; // = Verbindungen von Stadt names[curRow]
		queue.offer(new Integer(0));

		DefaultGraphCell[] cells = new DefaultGraphCell[size*2-1];
		//ArrayList cells = new ArrayList();
		System.out.println("--- Begin BFS ---");
		// ------
		for (int c = 0; c < size; c++) { // ,,,
			curRow = ((Integer) queue.poll()).intValue();
			System.out.print(c +" " +names.get(curRow));
			visited[curRow] = true;

			// ------ build cell
			// Create vertex with the given name
			if (cells[curRow] == null)
			cells[curRow] = createVertex((String) names.get(curRow));
			
			for (int i = 0; i < size; i++) {
				if (graph[curRow][i] == 1 && visited[i] == false) { // Verbindung
					queue.offer(new Integer(i));
					visited[i] = true;
					// new
					cells[i] = createVertex((String) names.get(i));
					// edge
					cells[i+size-1] = createEdge(cells[curRow], cells[i]);
					System.out.print(" " +cells[i].toString());
					//curCell.add(newCell);
				}
			}
			System.out.println();
		}
		// ------
		System.out.println("--- End BFS ---");
		return cells;
	}

	public static DefaultGraphCell createVertex(String name) {
		// Create vertex with the given name
		DefaultGraphCell cell = new DefaultGraphCell(name);

		Random rand = new Random();

		// Set bounds
		GraphConstants.setBounds(cell.getAttributes(), new Rectangle2D.Double(
				rand.nextInt(500), rand.nextInt(500), 80, 30));

		// Add a Floating Port
		cell.addPort();

		return cell;
	}

	public static DefaultEdge createEdge(DefaultGraphCell source,
			DefaultGraphCell target) {
		// Create Edge
		DefaultEdge edge = new DefaultEdge();
		// Fetch the ports from the new vertices, and connect them with the edge

		/*
		 * edge.getAttributes().applyMap(createEdgeAttributes()); public Map
		 * createEdgeAttributes() { Map map = new Hashtable(); // Add a Line End
		 * Attribute GraphConstants.setLineEnd(map,
		 * GraphConstants.ARROW_SIMPLE); // Add a label along edge attribute
		 * GraphConstants.setLabelAlongEdge(map, true);
		 * GraphConstants.setLineStyle(map, GraphConstants.STYLE_BEZIER);
		 * [b]GraphConstants.setRouting(map,
		 * GraphConstants.[u]ROUTING_SIMPLE[/u]);[/b] return map; }
		 */

		edge.setSource(source.getChildAt(0));
		edge.setTarget(target.getChildAt(0));
		// ((DefaultPort)target.getChildAt(0)).addEdge(edge);
		// ((DefaultPort)source.getChildAt(0)).addEdge(edge);

		// cells[1] = edge;

		// Set Arrow Style for edge
		int arrow = GraphConstants.STYLE_SPLINE;
		GraphConstants.setLineEnd(edge.getAttributes(), arrow);
		GraphConstants.setEndFill(edge.getAttributes(), true);

		return edge;
	}
}
