package treeplus.tree;

import java.awt.Color;
import java.awt.Point;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.Map;
import java.util.Queue;
import java.util.Random;
import java.util.concurrent.LinkedBlockingQueue;

import javax.swing.BorderFactory;
import javax.swing.border.Border;

import org.jgraph.graph.DefaultEdge;
import org.jgraph.graph.DefaultGraphCell;
import org.jgraph.graph.DefaultPort;
import org.jgraph.graph.GraphConstants;


public class BFS {

	public static HashMap levelMapping = new HashMap();


	
	public static DefaultGraphCell[] buildTree (TreePlusGraph treePlusGraph, int root, Object[] roots) {
		
		Queue queue = new LinkedBlockingQueue();	
		int[][] graph = treePlusGraph.getMatrix();
		ArrayList names = treePlusGraph.getNames();
		int size = names.size();
		boolean[] visited = new boolean[size]; // init false
		CellStruct[] cellStructs = new CellStruct[size];
		int level = 0;
		int[] isChild = new int[size];

		int curRow; // = Verbindungen von Stadt names[curRow]
		queue.offer(new Integer(root));

		DefaultGraphCell[] cells = new DefaultGraphCell[size*2-1];
		//ArrayList cells = new ArrayList();
		System.out.println("--- Begin BFS ---");
		
		// ------
		for (int c = 0; c < size; c++) { // ,,,
			curRow = ((Integer) queue.poll()).intValue();
			System.out.print(c +" " +names.get(curRow) +":" +curRow);
			visited[curRow] = true;

			// ------ build cell
			// Create vertex with the given name
			if (cells[curRow] == null) {
				cells[curRow] = createVertex((String) names.get(curRow));
				cellStructs[curRow] = new CellStruct(level, 0);
				levelMapping.put(new Integer(level), new Integer(1));
			}
			
			boolean newLevel = false;
			int localLevelCount = 0;
			
			// Verbindungen suchen
			for (int i = 0; i < size; i++) {
				if (graph[curRow][i] == 1 && visited[i] == false) { // Verbindung
					if (newLevel == false) {
						level++;
						newLevel= true;
					}
					
					queue.offer(new Integer(i));
					visited[i] = true;
					// new vertex
					cells[i] = createVertex((String) names.get(i));
					isChild[i] = 1;
					cellStructs[i] = new CellStruct(level, localLevelCount++);
					levelMapping.put(new Integer(level), new Integer(localLevelCount + 1));
					// edge
					cells[i+size-1] = createEdge(cells[curRow], cells[i]);
					System.out.print(" " +cells[i].toString());
				}
			}
			System.out.println();
		}
		// ------
		for (int i = 0; i < cells.length; i++) {
			if (cells[i] == null) {
				cells[i] = new DefaultGraphCell();
			}
		}
		for (int i = 0; i < cellStructs.length; i++) {
			GraphConstants.setBounds(cells[i].getAttributes(), 
					new Rectangle2D.Double(cellStructs[i].level*120,calcY(cellStructs[i].count, 
							((Integer)levelMapping.get(new Integer(cellStructs[i].level))).intValue()), 80, 30));

		}
		
		ArrayList rootObjs = new ArrayList();
		for (int i = 0; i < size; i++) {
			if (isChild[i] == 0) {
				rootObjs.add(cells[i]);
				System.out.println("Root found: " +i +cells[i].toString());
			}
		}
		roots = rootObjs.toArray(new Object[]{});
		
		System.out.println("--- End BFS ---");
		return cells;
	}
	
	
	private static int calcY(int count, int countForLevel) {
		int space = (700/countForLevel);
		int y = count*space + space/2;
		return y;
	}
	
	
	/**
	 * CREATE VERTEX
	 * @param name
	 * @return
	 */
	public static DefaultGraphCell createVertex(String name) {
		// Create vertex with the given name
		DefaultGraphCell cell = new DefaultGraphCell(name);
		
		Random rand = new Random();
		
		// Set bounds
		GraphConstants.setBounds(cell.getAttributes(), new Rectangle2D.Double(0, 0, 80, 30));
		GraphConstants.setBackground(cell.getAttributes(), Color.RED);
		GraphConstants.setBorder(cell.getAttributes(), BorderFactory.createRaisedBevelBorder());

		
		// Add a Floating Port
		cell.addPort();
		
		return cell;
	}
	
	/**
	 * CREATE EDGE
	 * @param source
	 * @param target
	 * @return
	 */
	public static DefaultEdge createEdge(DefaultGraphCell source, DefaultGraphCell target) {
		// Create Edge
		DefaultEdge edge = new DefaultEdge();
		// Fetch the ports from the new vertices, and connect them with the edge
				
		edge.setSource(source.getChildAt(0));
		edge.setTarget(target.getChildAt(0));
		
		edge.getAttributes().applyMap(createEdgeAttributes()); 
		
//		((DefaultPort)target.getChildAt(0)).addEdge(edge);
//		((DefaultPort)source.getChildAt(0)).addEdge(edge);

	//	cells[1] = edge;

		// Set Arrow Style for edge
		int arrow = GraphConstants.ARROW_TECHNICAL;
		
//		Point a = new Point (50,50);
//		Point b = new Point (100, 100);
//		LinkedList ll = new LinkedList();
//		ll.add(a);
//		ll.add(b);
//		
//		GraphConstants.setPoints(edge.getAttributes(),ll);
//		GraphConstants.setLineStyle(edge.getAttributes(),GraphConstants.STYLE_SPLINE);
//		GraphConstants.setRouting(edge.getAttributes(), GraphConstants.ROUTING_DEFAULT);
		GraphConstants.setLineEnd(edge.getAttributes(), arrow);
		GraphConstants.setBendable(edge.getAttributes(), true);
		GraphConstants.setEndFill(edge.getAttributes(), true);
		
		return edge;
	}
	
	
	  public static Map createEdgeAttributes() 
      { 
         Map map = new Hashtable(); 
         // Add a Line End Attribute 
         GraphConstants.setLineEnd(map, GraphConstants.ARROW_SIMPLE); 
         // Add a label along edge attribute 
         GraphConstants.setLabelAlongEdge(map, true); 
         GraphConstants.setLineStyle(map, GraphConstants.STYLE_SPLINE); 
//         GraphConstants.setRouting(map, GraphConstants.ROUTING_SIMPLE); 
         return map; 
      } 
}
